<!DOCTYPE html>
<html lang="it">
    <head>
        <meta charset="UTF-8"/>
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <title>Thor</title>
        <link rel="stylesheet" href="{{ url_for("static", filename="style.css") }}"/>
    </head>
    <body>
        <h1>
            Thor
        </h1>
        <div class="container">
            {% block contents %}
            {% endblock %}
        </div>
    </body>
</html>
